/*
 * JBoss, Home of Professional Open Source
 * Copyright 2005, JBoss Inc., and individual contributors as indicated
 * by the @authors tag. See the copyright.txt in the distribution for a
 * full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */
package org.jbpm.test.query;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.jbpm.api.ProcessDefinition;
import org.jbpm.api.ProcessInstance;
import org.jbpm.api.history.HistoryActivityInstance;
import org.jbpm.api.history.HistoryActivityInstanceQuery;
import org.jbpm.pvm.internal.util.Clock;
import org.jbpm.test.JbpmTestCase;
import org.jbpm.test.assertion.QueryAssertions;
import org.jbpm.api.Execution;


/**
 * @author Tom Baeyens
 */
public class ForkHistoryQueryTest extends JbpmTestCase {
  

  public void testSimpleQuery() {
		
		deployTestProcess();
		 ProcessInstance processInstance = executionService.startProcessInstanceByKey("ForkAssumptionTest");
		 assertNotNull(processInstance.findActiveExecutionIn("state1"));
		 processInstance = executionService.signalExecutionById(processInstance.getId());
		 Execution node = processInstance.findActiveExecutionIn("state2");
		 assertNotNull(node);
		 assertNotNull(processInstance.findActiveExecutionIn("state3"));
		 processInstance = executionService.signalExecutionById(node.getId());
		 node = processInstance.findActiveExecutionIn("state3");
		 processInstance = executionService.signalExecutionById(node.getId());
		 node = processInstance.findActiveExecutionIn("state4");
		 assertNotNull(node);
		 executionService.signalExecutionById(node.getId());
		  List<HistoryActivityInstance> o = historyService.createHistoryActivityInstanceQuery().executionId(processInstance.getId()).list();
		  assertEquals(4,o.size());  //Comes back 2.  Only state1 and state4 are on the list
		  assertEquals("state1",o.get(0).getActivityName());
		  assertEquals("state4",o.get(1).getActivityName());
		  
		  HistoryActivityInstance oo = historyService.createHistoryActivityInstanceQuery().activityName("state2").uniqueResult();
		  assertFalse(processInstance.getId()== oo.getExecutionId()); 
  }
  
 
  

  
  /**
   * Returns deployment db id
   */
  private String deployTestProcess() {
    return deployJpdlXmlString(
"<process name='ForkAssumptionTest'>"+
"   <start name='start1'> <transition to='state1'/> </start>" +
" <state name='state1'><transition to='fork1' /></state>" +
"   <state name='state2'><transition to='join1'/></state>"+
"   <state name='state3'><transition  to='join1' /></state>"+
"   <state name='state4'> <transition to='end1' /></state>"+
"    <fork name='fork1'><transition  to='state2'/><transition  to='state3' /></fork>"+
"   <join name='join1'><transition to='state4' /></join>"+
"   <end name='end1'/>"+
"</process>"
          );
  }

  
  
}
